# 《从 Paxos 到 ZooKeeper：分布式一致性原理与实践》 倪超 著
## 书评
## 目录
## 第 1 章 分布式架构 
### ACID
- `原子性（Atomicity）`：全部成功执行/全部不执行
- `一致性（Consistency）`：事务的执行不能破坏数据库数据的完整性和一致性。
- `隔离性（Isolation）`：并发环境中，并发的事务是相互隔离的。
- `持久性（Durability）`：一个事务一旦提交成功，它对数据库中对应数据的状态变更就应该是永久性的。
### 分布式事务
为了保证分布式应用程序的可靠性，分布式事务是无法回避的。
### CAP 理论
- `一致性（C：Consistency）`：在分布式环境中，数据在多个副本之间能否保持一致的特性。
- `可用性（A：Availability）`：可用性是指系统提供的服务必须一直处于可用的状态。（有限的时间内 和 返回结果）
- `分区容错性（P：Partition tolerance）`：分布式系统在遇到任何网络分区故障的时候，仍然需要能够保证对外提供满足一致性和可用性的服务，除非是整个网络环境都发生了故障。
### BASE 理论
- `基本可用（Basically Available）`：
- `软状态（Soft state）`：弱状态也称为软状态，和硬状态相对，是指允许系统中的数据存在中间状态，并认为该中间状态的存在不会影响系统的整体可用性，即允许系统在不同节点的数据副本之间进行数据同步的过程存在延时。
- `最终一致性（Eventually consistent）`：最终一致性强调的是系统中所有的数据副本，在经过一段时间的同步后，最终能够达到一个一致的状态。因此，最终一致性的本质是需要系统保证最终数据能够达到一致，而不需要实时保证系统数据的强一致性。
## 第 2 章 一致性协议
### 2PC Two-Phase Commit
阶段一：提交事务请求

1. `事务询问`：协调者向所有的参与者发送事务内容，询问是否可以执行事务提交操作，并开始等待各参与者的响应。
2. `执行事务`：各参与者节点执行事务操作，并将 Undo 和 Redo 信息记入事务日志中。
3. `各参与者向协调者反馈事务询问的响应`：如果参与者成功执行了事务操作，那么就反馈给协调者 Yes 响应，表示事务可以执行；如果参与者没有成功执行事务。那么就反馈给协调者 No 响应，表示事务不可以执行。

阶段二：执行事务提交

- 可能一：执行事务提交

    假如协调者从所有的参与者获得的反馈都是 Yes 响应，那么就会执行事务提交。
    1. `发送提交请求`：协调者向所有参与者节点发出 Commit 请求。
    2. `事务提交`：参与者接收到 Commit 请求后，会正式执行事务提交操作，并在完成提交之后释放在整个事务执行期间占用的事务资源。
    3. `反馈事务提交结果`：参与者在完成事务提交后，向协调者发送 Ack 消息。
    4. `完成事务`：协调者接收到所有参与者反馈的 Ack 消息后，完成事务。

- 可能二：中断事务

    假如任何一个参与数向协调者反馈了 No 响应，或者在等待超时之后，协调者尚无法接收到所有参与者的反馈响应，那么就会中断事务。
    1. `发送回归请求`：协调者向所有参与者节点发出 RollbAck 请求。
    2. `事务回滚`：参与者接收到 RollbAck 请求后，会利用其在阶段一种记录的 Undo 信息来执行事务回滚操作，并在完成回滚之后释放在整个事务执行期间占用的资源。
    3. `反馈事务回滚请求`：参与者在完成事务回滚之后，向协调者发送 Ack 消息。
    4. `中断事务`：协调者接收到所有参与者反馈的 Ack 消息后，完成事务中断。

优缺点

- 优点：原理简单，实现方便
- 缺点：同步阻塞、单点问题、脑裂、太过保守
### 3PC Three-Phase Commit
阶段一：CanCommit

1. `事务询问`：协调者向所有的参与者发送一个包含事务内容的 CanCommit 请求，询问是否可以执行事务提交操作，并开始等待各参与者的响应。
2. `各参与者向协调者反馈事务询问的响应`：参与者在接收到协调者的 CanCommit 请求后，正常情况下，如果其自身任务可以顺利执行事务，那么会反馈 Yes 响应，并进入预备状态，否则反馈 No 响应。

阶段二：PreCommit

在阶段二中，协调者会根据各参与者的反馈情况来决定是否可以进行事务的 PreCommit 操作，正常情况下，包含两种可能。

- 可能一：执行事务预提交

    假如协调者从所有的参与者获得的反馈都是 Yes 响应，那么就会执行事务预提交。
    1. `发送预提交请求`：协调者向所有参与者节点发出 PreCommit 请求，并进入 Prepared 阶段。
    2. `事务预提交`：参与者接收到 PreCommit 请求后，会执行事务操作，并将 Undo 和 Redo 信息记录到事务日志中。
    3. `各参与者向协调者反馈事务执行的响应`：如果参与者成功执行了事务操作，那么就会反馈给协调者 Ack 响应，同时等待最终的指令：提交（commit）或中止（abort）。

- 可能二：中断事务

    假如任何一个参与者向协调者反馈了 No 响应，或者在等待超时之后，协调者尚无法接收到所有参与者的反馈响应，那么就会中断事务。
    1. `发送中断请求`：协调者向所有参与者节点发出 abort 请求
    2. `中断事务`：无论是收到来自协调者的 abort 请求，或者是在等待协调者请求过程中出现超时，参与者都会中断事务。

阶段三：doCommit

该阶段将进行真正的事务提交，会存在以下两种可能的情况。

- 可能一：执行提交
    1. `发送提交请求`：进入这一阶段，**假设协调者处于正常工作状态**，并且它接收到了来自所有参与者的 Ack 响应，那么它将从“预提交”状态转换到“提交”状态，并向所有的参与者发送 doCommit 请求
    2. `事务提交`：参与者接收到 doCommit 请求后，会正式执行事务提交操作，并在完成提交之后释放整个事务执行期间占用的事务资源。
    3. `反馈事务提交结果`：参与者在完成事务提交之后，向协调者发送 Ack 消息。
    4. `完成事务`：协调者接收到所有参与者反馈的 Ack 消息后，完成事务。

- 可能二：中断事务

    进入这一阶段，**假设协调者处于正常工作状态**，并且有任意一个参与者向协调者反馈了 No 响应，或者在等待超时之后，协调者尚无法接收到所有参与者的反馈响应，那么就会中断事务。
    1. `发送中断请求`：协调者向所有的参与者节点发送 abort 请求。
    2. `事务回滚`：参与者接收到 abort 请求后，会利用其在阶段二中记录的 Undo 信息来执行事务回滚操作，并在完成回滚之后释放在整个事务执行期间占用的资源。
    3. `反馈事务回滚结果`：参与者在完成事务回滚之后，向协调者发送 Ack 消息。
    4. `中断事务`：协调者接收到所有参与者反馈的 Ack 消息后，中断事务。

需要注意的是，一旦进入阶段三，可能会存在以下两种故障。
- **协调者出现问题**
- **协调者和参与者直接的网络出现故障**

无论出现哪种情况，最终都会导致参与者无法及时接受到来自协调者的 doCommit 或是 abort 请求，针对这样的异常情况，参与者都会在等待超时之后，继续进行事务提交。

优缺点

- 优点：相较于二阶段提交协议，三阶段提交协议最大的优点就是降低了参与者的阻塞范围，并且能够在出现单点故障后继续达成一致。
- 缺点：三阶段提交协议在去除阻塞的同时也引入了新的问题，那就是在参与者接收到 preCommit 消息后，如果出现网络分区，此时协调者所在的节点和参与者无法进行正常的网络通信，在这种情况下，该参与者依然会进行事务的提交，这必然出现数据的不一致性。
### Paxos 算法

`“拜占庭将军问题”`：拜占庭帝国有许多支军队，不同军队的将军之间必须制定一个统一的行动计划，从而做出进攻或者撤退的决定，同时，哥哥将军在地理上都是被分隔开来的，只能依靠军队的通讯员来执行通讯。然而，在所有的通讯员中可能会存在叛徒，这些叛徒可以任意篡改消息，从而达到欺骗将军的目的。

在古希腊有一个叫做 paxos 的小岛，岛上采用议会的形式来通过法令，议会中的议员通过信使进行消息的传递。值得注意的是，议员和信使都是兼职的，他们损失有可能会离开议会厅，并且信使可能会重复的传递消息，也可能一区不复返。因此，议会协议要保证在这种情况下法令仍然能够正确的产生，并且不会出现冲突。
## 第 3 章 Paxos 的工程实践
## 第 4 章 ZooKeeper 与 Paxos
ZAB
## 第 5 章 使用 ZooKeeper
## 第 6 章 ZooKeeper 的典型应用场景
## 第 7 章 ZooKeeper 技术内幕
## 第 8 章 ZooKeeper 运维